// Activator.vala

/*
This file is part of MPRIS Applet Base

GNOME MPRIS Base is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

In addition, as a special exception, The developers of the GNOME MPRIS Base grants permission for non-GPL compatible GStreamer plugins to be used and distributed together with GStreamer and GNOME MPRIS Base. This permission is above and beyond the permissions granted by the GPL license by which GNOME MPRIS Base is covered. If you modify this code, you may extend this exception to your version of the code, but you are not obligated to do so. If you do not wish to do so, delete this exception statement from your version.

GNOME MPRIS Base is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
*/

using GLib;

namespace MprisApp {
	public class Activator {
		private weak MainControl app;
		private MasterDBus? bus;
		private PlayersList? _players = null;
		public PlayersList? players {
			get { return _players; }
		}
		private PlayersList? _current_player = null;
		public PlayersList? current_player {
			get { return _current_player; }
			set { _current_player = value; }
		}
		
		public Activator (MainControl main_control) {
			app = main_control;
		}
		
		public void initialize () {
			try {
				bus = Bus.get_proxy_sync (BusType.SESSION, "org.freedesktop.DBus",
				  "/org/freedesktop/DBus");
				repeat_list ();
			}
			catch(IOError e) {
				report_err (e.message);
			}
		}
		
		private const string prefix = "org.mpris.MediaPlayer2.";
		private void list_names_reply_cb (string[] addresses) {
			uint8 count = (uint8)prefix.length;
			foreach (string address in addresses) {
				uint8 cur_count = (uint8) address.length;
				if (cur_count > count) {
					uint8 i = 0;
					for(; i < count; i++)
						if (address[i] != prefix[i]) {
							i = count + 1;// make it's break, but not confuse the "if" below.
						}
					if (i == count)
						add_working_player (address);
				}
			}
		}
		
		private void add_working_player (string address) {
			if (PlayersList.find_by_address (_players, address) == null) {
				report ("Found media player: " + address);
				void* iter = app.make_new_iter ();
				PlayersList.append (ref _players, address, iter);
			}
		}
		
		internal inline void* send_icon_request (string source) {
			return app.make_new_icon (source);
		}
		
		internal inline void on_player_ready (Player player) {
			app.on_player_ready (player, (void*)player == (void*)_current_player);
		}
		
		internal inline void on_player_dead (PlayersList player_list) {
			if (PlayersList.remove (ref _players, player_list)) {
				report ("Removing media player: " + player_list.current.address);
				if ((void*)player_list == (void*)current_player) {
					if (_players == null)
						current_player = null;
					else
						current_player = players;
					app.on_player_dead (player_list.current, true);
					return;
				}
				//else
				app.on_player_dead (player_list.current, false);
			}
		}
		
		internal inline void on_player_icon_ready (Player player) {
			app.on_player_icon_ready (player, (void*)player == (void*)_current_player);
		}

		private void repeat_list () {
			bus.list_names.begin ((obj, res) => {
				try {
					list_names_reply_cb (bus.list_names.end (res));
				} catch (IOError e) {
					report_err ("Can't list: " + e.message);
				} finally {
					repeat_list ();
				}
			});
		}
	}
}
