// Player.vala

/*
This file is part of MPRIS Applet Base

GNOME MPRIS Base is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

In addition, as a special exception, The developers of the GNOME MPRIS Base grants permission for non-GPL compatible GStreamer plugins to be used and distributed together with GStreamer and GNOME MPRIS Base. This permission is above and beyond the permissions granted by the GPL license by which GNOME MPRIS Base is covered. If you modify this code, you may extend this exception to your version of the code, but you are not obligated to do so. If you do not wish to do so, delete this exception statement from your version.

GNOME MPRIS Base is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
*/

using GLib;

namespace MprisApp {
	public class Player : GLib.Object {
		private weak PlayersList parent;
		private void* _iter;
		public void* iter {
			get { return _iter; }
		}
		
		private int got_count = 0;
		private static const int to_got = 2;
		public bool tl_ready = false;
		
		private string _address;
		public string address {
			get { return _address; }
		}
		private Mpris2.MediaPlayer mp;
		private Mpris2.Player p;
		private Mpris2.TrackList? tl;
		
		private void* _desktop_icon = null;
		public void* desktop_icon {
			get { return _desktop_icon; }
		}
		
		// MPRIS MediaPlayer properties
		public bool can_quit { get { return mp.CanQuit; } }
		public bool can_raise { get { return mp.CanRaise; } }
		public bool has_track_list { get { return mp.HasTrackList; } }
		public string identity { owned get { return mp.Identity; } }
		public string desktop_entry { owned get { return mp.DesktopEntry; } }
		public string[] supported_uri_schemes { owned get { return mp.SupportedUriSchemes; } }
		public string[] supported_mime_schemes { owned get { return mp.SupportedMimeSchemes; } }
		
		// MPRIS Player properties
		public PlaybackStatus playback_status {
			owned get {	return PlaybackStatus.from_up_low_string (p.PlaybackStatus); }
		}
		public LoopStatus loop_status { 
			owned get { return LoopStatus.from_up_low_string (p.LoopStatus);  }
		}
		public double rate { get { return p.Rate; } }
		public double minimum_rate { get { return p.MinimumRate; } }
		public double maximum_rate { get { return p.MaximumRate; } }
		public bool can_go_next { get { return p.CanGoNext; } }
		public bool can_go_previous { get { return p.CanGoPrevious; } }
		public bool can_play { get { return p.CanPlay; } }
		public bool can_pause { get { return p.CanPause; } }
		public bool can_seek { get { return p.CanSeek; } }
		private bool _can_control;
		public bool can_control { get { return _can_control; } }
		
		public Player (string address, void* iter) {
			_address = address;
			_iter = iter;
			Bus.get_proxy.begin<Mpris2.MediaPlayer> (BusType.SESSION, address, "/org/mpris/MediaPlayer2", 0, null, (obj, res) => {
				try {
					Mpris2.MediaPlayer _mp = Bus.get_proxy.end (res);
					if (_mp.HasTrackList)
						make_track_list ();
					string desktop_entry = _mp.DesktopEntry;
					if (desktop_entry != null)
						read_desktop_entry.begin (desktop_entry);
					report (desktop_entry);
					if (_mp.CanRaise)
						_mp.Raise ();
					cont_and_trigger_ready ();
					mp = _mp;
				} catch (IOError e) {
					parent.on_player_dead ();
					/*report_err ("Can't configure mp: " + Address + "Reason: " + e.message);
					message ("Can't configure mp: " + Address + "Reason: " + e.message);*/
				}
	}		);

			Bus.get_proxy.begin<Mpris2.Player> (BusType.SESSION, address, "/org/mpris/MediaPlayer2", 0, null, (obj, res) => {
				try {
					Mpris2.Player _p = Bus.get_proxy.end (res);
					_can_control = _p.CanControl;
					//check_playback ();
					//handle_playback ();
					// p.Pause();
					cont_and_trigger_ready ();
					//Idle.add (() => { regular_check (); return true; });
				} catch (IOError e) {
					parent.on_player_dead ();/*
					report_err ("Can't configure p: " + Address + "Reason: " + e.message);
					message ("Can't configure p: " + Address + "Reason: " + e.message);*/
				}
			});
		}

		private inline void cont_and_trigger_ready () {
			AtomicInt.inc (ref got_count);
			if (got_count == to_got) {
				parent.on_player_ready ();
			}
		}
		
		private void make_track_list () {
			Bus.get_proxy.begin<Mpris2.TrackList> (BusType.SESSION, address, "/org/mpris/MediaPlayer2", 0, null, (obj, res) => {
				try {
					tl = Bus.get_proxy.end (res);
					tl_ready = true;
				} catch (IOError e) {
					parent.on_player_dead ();/*
					report_err ("Can't configure tl: " + Address + "Reason: " + e.message);
					message ("Can't configure tl: " + Address + "Reason: " + e.message);*/
				}
			});
		}
		
		/*private void read_skip_space (string input, uint length, ref uint i) {
			for (; i < length; i++)
				if (input[i] != ' ')
					return;
		}*/
		
		private static const string icon_str = "Icon=";
		private async void read_desktop_entry (string desktop_entry) {
			try {
				File file = File.new_for_path ("/usr/share/applications/" + desktop_entry + ".desktop");
				DataInputStream dis = new DataInputStream (file.read ());
				string? line = null;
			
				while ((line = yield dis.read_line_async (Priority.DEFAULT)) != null) {
					uint length = line.length;
					uint i = 0;
					if (length < icon_str.length)
						continue;
	            	for (; i < icon_str.length; i++)
	            		if (line[i] != icon_str[i])
							i = length;
					if(i < length) {
						/*read_skip_space (line, length, ref i);
						if (line[i] != '=')
							continue;
						i++;*/
						string icon_name = line[i:length];
						report ("Found icon: " + icon_name);

						_desktop_icon = parent.send_icon_request (icon_name);
						parent.on_player_icon_ready ();
					}
	        	}
			} catch (Error e) {
	        	error (e.message);
	    	}
	    }
		/*
		public void regular_check () {
		}

		private void handle_playback () {
			switch (playback_status) {
				case PlaybackStatus.Playing :
					parent.change_stop_btn_sensitive (true);
					parent.switch_play_btn (false);
					break;
				case PlaybackStatus.Paused :
					parent.change_stop_btn_sensitive (true);
					parent.switch_play_btn (true);
					break;
				case PlaybackStatus.Stopped :
					parent.change_stop_btn_sensitive (false);
					parent.switch_play_btn (true);
					break;
				default : report_err ("Can't configure p: " + Address + "Reason: " + "Not understandable PlaybackStatus");
					break;
			}
		}
		
		public void regular_handle () {
			handle_playback ();
		}
		*/
		
		// MPRIS MediaPlayer methods
		public void quit () throws IOError {
			mp.Quit ();
		}
		public void raise () throws IOError {
			mp.Raise ();
		}
		
		// MPRIS Player methods
		public void next() throws IOError {
			p.Next ();
		}
		public void previous() throws IOError {
			p.Previous ();
		}
		public void pause () throws IOError {
			p.Pause ();
		}
		public void play () throws IOError {
			p.Play ();
		}
		public void play_pause () throws IOError {
			p.PlayPause ();
		}
		public void stop () throws IOError {
			p.Stop ();
		}
		public void seek (int64 offset) throws IOError {
			p.Seek (offset);
		}
		public void set_position (string track_id, int64 position) throws IOError {
			p.SetPosition (track_id, position);
		}
		public void open_uri (string uri) throws IOError {
			p.OpenURI (uri);
		}
	}
}
