// PlayersList.vala

/*
This file is part of MPRIS Applet Base

GNOME MPRIS Base is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.

In addition, as a special exception, The developers of the GNOME MPRIS Base grants permission for non-GPL compatible GStreamer plugins to be used and distributed together with GStreamer and GNOME MPRIS Base. This permission is above and beyond the permissions granted by the GPL license by which GNOME MPRIS Base is covered. If you modify this code, you may extend this exception to your version of the code, but you are not obligated to do so. If you do not wish to do so, delete this exception statement from your version.

GNOME MPRIS Base is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
*/

using GLib;

namespace MprisApp {
	public class PlayersList : GLib.Object {
		private weak Activator activator;
		
		//public static int atomic_lock = 0;
		private Player _current;
		public Player current {
			get { return _current; }
		}
		private PlayersList? _next;
		public PlayersList? next { 
			get { return _next; } 
		}

		public PlayersList (string address, void* iter) {
			_current = new Player (address, iter);
		}
		
		internal inline void* send_icon_request (string source) {
			return activator.send_icon_request (source);
		}
		
		internal inline void on_player_ready () {
			activator.on_player_ready (current);
		}
		
		internal inline void on_player_dead () {
			activator.on_player_dead (this);
		}
		
		internal inline void on_player_icon_ready () {
			activator.on_player_icon_ready (current);
		}

		public static PlayersList? last (PlayersList? list) {
			PlayersList? cur = list;
			if (cur == null)
				return null;
			PlayersList? cur__next = list._next;
			while (cur__next != null) {
				cur = cur__next;
				cur__next = cur__next._next;
			}
			return cur;
		}

		public static PlayersList append (ref PlayersList? list, string address, void* iter) {
			PlayersList output = new PlayersList(address, iter);
			if (list == null)
				list = output;
			else
				last (list)._next = output;
			return output;
		}

		/*public void foreach (Func func) {
			PlayersList? list = this;
			while (list != null) {
				func (list);
				list = list._next;
			}
		}*/

		public static bool remove (ref PlayersList? list, PlayersList to_remove) {
			if (list == null)
				return false;
			//else
			PlayersList current = list;
			PlayersList? _next = current._next;
			if (current == to_remove) {
				list = _next;
				return true;
			}
			while (_next != null) {
				current = _next;
				_next = _next._next;
				if (current == to_remove) {
					list = _next;
					return true;
				}
			}
			return false;
		}

		public static PlayersList? find_by_address (PlayersList? list, string data) {
			while (list != null) {
				if (list.current.address == data)
					return list;
				list = list._next;
			}
			//else
			return null;
		}
	}
}
